@@ -196,7 +196,7 @@ sub WriteCR2($$$)
 
     # check CR2 signature
     return 0 if $$dataPt !~ /^.{8}CR\x02\0/s and
-        $exifTool->Warn("Unsupported Canon RAW file. May cause problems if rewritten", 1);
+        $exifTool->Error("Unsupported Canon RAW file. May cause problems if rewritten", 1);
     # CR2 has a 16-byte header
     $$dirInfo{NewDataPos} = 16;
     my $newData = $exifTool->WriteDirectory($dirInfo, $tagTablePtr);
@@ -205,11 +205,13 @@ sub WriteCR2($$$)
     if (length($newData)) {
         # build 16 byte header for Canon RAW file
         my $header = substr($$dataPt, 0, 16);
-        # last 4 bytes is pointer to last IFD
+        # set IFD0 pointer (may not be 16 if edited by PhotoMechanic)
+        Set32u(16, \$header, 4);
+        # last 4 bytes of header is pointer to last IFD
         Set32u($$dirInfo{LastIFD}, \$header, 12);
         Write($outfile, $header, $newData) or return 0;
         undef $newData;     # free memory
-        
+
         # copy over image data now if necessary
         if (ref $$dirInfo{ImageData}) {
             $exifTool->CopyImageData($$dirInfo{ImageData}, $outfile) or return 0;
@@ -497,26 +499,32 @@ sub WriteCRW($$)
     my $rtnVal = 0;
     my ($buff, $err, $sig);
 
-    $raf->Read($buff,2) == 2 or return 0;
-    SetByteOrder($buff)      or return 0;
-    $raf->Read($buff,4) == 4 or return 0;
-    $raf->Read($sig,8) == 8  or return 0;   # get file signature
-    $sig eq 'HEAPCCDR'       or return 0;   # validate signature
+    $raf->Read($buff,2) == 2    or return 0;
+    SetByteOrder($buff)         or return 0;
+    $raf->Read($buff,4) == 4    or return 0;
+    $raf->Read($sig,8) == 8     or return 0;   # get file signature
+    $sig =~ /^HEAP(CCDR|JPGM)/  or return 0;   # validate signature
+    my $type = $1;
     my $hlen = Get32u(\$buff, 0);   # get header length
 
+    if ($exifTool->{DEL_GROUP}) {
+        if ($type eq 'CCDR') {
+            $exifTool->Error("Can't delete groups in CRW file");
+            return 0;
+        } elsif ($exifTool->{DEL_GROUP}->{MakerNotes}) {
+            ++$exifTool->{CHANGED};
+            return 1;
+        }
+    }
+
     # write header
-    $raf->Seek(0, 0)         or return 0;
+    $raf->Seek(0, 0)            or return 0;
     $raf->Read($buff, $hlen) == $hlen or return 0;
     Write($outfile, $buff) or $err = 1;
 
-    $raf->Seek(0, 2)         or return 0;   # seek to end of file
+    $raf->Seek(0, 2)            or return 0;   # seek to end of file
     my $filesize = $raf->Tell() or return 0;
 
-    if ($exifTool->{DEL_GROUP}) {
-        $exifTool->Error("Can't delete groups in CRW file");
-        return 0;
-    }
-
     # build directory information for main raw directory
     my %dirInfo = (
         DataLen  => 0,
@@ -528,7 +536,6 @@ sub WriteCRW($$)
         OutFile  => $outfile,
         OutPos   => $hlen,
     );
-
     # process the raw directory
     my $tagTablePtr = Image::ExifTool::GetTagTable('Image::ExifTool::CanonRaw::Main');
     my $msg = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
